#!/bin/sh

USER=vscan
GROUP=vscan

samefile() {
  if [ "`stat -L -c %d.%i $1`" = "`stat -L -c %d.%i $2`" ]; then
    return 0
  fi >/dev/null 2>&1
  return 1
}

addfile() {
  cp -af $1 $ROOTPATH/$1
}

addbin() {
  n=0
  for file in "$@"; do
    if [ -f $file ]; then
      n=`expr $n + 1`
      cp -f $file $ROOTPATH/$file
      if ldd $file > /dev/null 2>&1 ; then
        for i in `ldd $file 2>/dev/null | awk '/=> *\(/ {next} {print $3}' \
            | sed -e 's/^\/lib\/tls/\/lib/g' -e 's/^\/lib\/i[456]86/\/lib/g' \
            -e 's/^\/usr\/lib\/sse2/\/usr\/lib/g'`
        do
          addbin $i
        done
      fi
    fi
  done
  if [ "$n" = "0" ]; then
    echo "Warning: Commands $@ are missing."
  fi
}

addconf() {
  for file in "$@"; do
    if [ -f "$file" ]; then
      if [ -f "$ROOTPATH$file" ]; then
        samefile "$file" "$ROOTPATH$file" \
          || echo "Warning: $file found in $ROOTPATH, update skipped!"
      else
        addfile $file
      fi
    fi
  done
}

adduser() {
  grep ^$1 /etc/passwd >> $ROOTPATH/etc/passwd
}

addgroup() {
  grep ^$1 /etc/group | sed -e 's/:[^:]*$/:/g' >> $ROOTPATH/etc/group
}

showhelp() {
  echo "Usage:"
  echo "  `basename $0` [chroot_directory] [--auto] [--help] \\"
  echo "       [--clamav] [--kav] [--bdc] \\"
  echo "       [--bogofilter] [--qsf]"
  echo ""
  echo "Where:"
  echo "  --auto       - select all options automatically"
  echo "  --help       - show this help"
  echo "  --clamav     - install clamav"
  echo "  --nod        - install nod32"
  echo "  --avg        - install avglinux"
  echo "  --kav        - install kaspersky"
  echo "  --bdc        - install bitdefender"
  echo "  --bogofilter - install bogofilter"
  echo "  --qsf        - install quick spam filter"
  exit
}

if [ "$1" ]; then
  if echo "$1" | grep -q ^/. >/dev/null; then
    export ROOTPATH="$1"
    shift 1
  else
    eval `grep ^CHROOT= /etc/sagator.conf`
    if [ -z "$CHROOT" ]; then
      echo "Chroot not set, skipping mkchroot!"
      exit
    fi
    export ROOTPATH="$CHROOT"
    echo "CHROOT=$CHROOT"
  fi
else
  showhelp
fi

for i in dev etc bin sbin lib tmp/quarantine usr/lib usr/bin usr/sbin usr/share var/log/sagator var/lib proc; do
  mkdir -p "$ROOTPATH"/$i
done
ln -sf "$ROOTPATH/var/log/sagator" /var/log/

cat > "$ROOTPATH/etc/nsswitch.conf" <<EOF
passwd:     files
shadow:     files
group:      files

hosts:      files,dns

ethers:     files
netmasks:   files
networks:   files
protocols:  files
rpc:        files
services:   files
EOF

cat > "$ROOTPATH/etc/hosts" <<EOF
127.0.0.1	localhost.localdomain localhost
EOF

rm -f "$ROOTPATH/etc/passwd"
adduser root
adduser $USER

rm -f "$ROOTPATH/etc/group"
addgroup root
addgroup $GROUP

unset CLAMAV NOD AVG BOGOFILTER QSF BDC KAV
for arg in "$@"; do
  if [ "$arg" = "--auto" ]; then
    if [ -e /usr/lib/libclamav.so.1 ]; then
      CLAMAV="YES"
    fi
    if [ -x /opt/nod32/nod32d -o -x /usr/local/nod32/nod32d -o -d /etc/nod32 ]; then
      NOD="YES"
    fi
    if [ -x /opt/bdc/bdc ]; then
      BDC="YES"
    fi
    if [ -x /opt/kav/bin/kavscanner ]; then
      KAV="YES"
    fi
    if [ -x /usr/local/bin/avgscan ]; then
      AVG="YES"
    fi
    if [ -x /usr/bin/bogofilter ]; then
      BOGOFILTER="YES"
    fi
    if [ -x /usr/bin/qsf ]; then
      QSF="YES"
    fi
  elif [ "$arg" = "--help" ]; then
    showhelp
  elif [ "$arg" = "--clamav" ]; then
    CLAMAV="YES"
  elif [ "$arg" = "--nod" ]; then
    NOD="YES"
  elif [ "$arg" = "--avg" ]; then
    AVG="YES"
  elif [ "$arg" = "--kav" ]; then
    KAV="YES"
  elif [ "$arg" = "--bdc" ]; then
    BDC="YES"
  elif [ "$arg" = "--bogofilter" ]; then
    BOGOFILTER="YES"
  elif [ "$arg" = "--qsf" ]; then
    QSF="YES"
  else
    addbin "$arg"
  fi
done

if [ "$CLAMAV" ]; then
  adduser clamav
  addgroup clamav
  addbin /usr/bin/clam*scan
  addbin /usr/sbin/clamd
  if [ -d /etc/clamav ]; then
    mkdir -p "$ROOTPATH/etc/clamav"
    addconf /etc/clamav/clamav.conf
  else
    addconf /etc/clamav.conf
  fi
  mkdir -p "$ROOTPATH/var/lib/clamav" "$ROOTPATH/var/log/clamav" \
    "$ROOTPATH/var/run/clamav"
  samefile "/var/lib/clamav" "$ROOTPATH/var/lib/clamav" \
    || ( cp -f /var/lib/clamav/* "$ROOTPATH/var/lib/clamav";
         echo "Run: /usr/bin/freshclam --datadir=$ROOTPATH/var/lib/clamav"
       )
  [ "`id -u`" = "0" ] && chown -R clamav.$GROUP "$ROOTPATH/var/lib/clamav" \
    "$ROOTPATH/var/run/clamav" "$ROOTPATH/var/log/clamav"
  chmod 770 "$ROOTPATH/var/run/clamav"
fi
if [ "$NOD" ]; then
  # OLD NOD
  for nodpath in /opt/nod32 /usr/local/nod32; do
    if [ -x $nodpath/nod32d ]; then
      samefile "$nodpath/nod32d" "$ROOTPATH$nodpath/nod32d" \
        || echo "Warning: Please reinstall NOD32 in $ROOTPATH$nodpath !"
    fi
  done
  # NOD2
  if [ -d /etc/nod32 ]; then
    mkdir -p "$ROOTPATH/etc/nod32"
    addconf /etc/nod32/*
    addbin /usr/sbin/nod32*
    samefile /var/lib/nod32 "$ROOTPATH/var/lib/nod32" \
      || cp -rf /var/lib/nod32 "$ROOTPATH/var/lib/"
  fi
fi
if [ "$AVG" ]; then
  mkdir -p "$ROOTPATH/usr/local/bin" "$ROOTPATH/usr/local/lib/avg7"
  addconf /etc/avg.conf
  addbin /usr/local/lib/avg7/avgscan
  addbin /usr/local/lib/avg7/avgupdate
  cp -f /usr/local/lib/avg7/*.* "$ROOTPATH/usr/local/lib/avg7/"
  chown -R $USER.$GROUP "$ROOTPATH/usr/local/lib/avg7"
  samefile "/usr/local/lib/avg7" "$ROOTPATH/usr/local/lib/avg7" \
    || echo "Warning: Do not forget to update AVG database in $ROOTPATH !"
fi
if [ "$BDC" ]; then
  mkdir -p "$ROOTPATH/opt/"
  cp -rf /opt/bdc "$ROOTPATH/opt/"
  for i in bdc bdcore.so bdupd.so libfn.so; do
    addbin /opt/bdc/$i
  done
  samefile "/opt/bdc" "$ROOTPATH/opt/bdc" \
    || echo "Warning: Do not forget to update BDC database in $ROOTPATH !"
  [ -f $ROOTPATH/proc/version ] \
    || echo "Warning: You need to mount /proc also as $ROOTPATH/proc !"
fi
if [ "$KAV" ]; then
  mkdir -p "$ROOTPATH/opt/"
  mkdir -p "$ROOTPATH/var/db/"
  mkdir -p "$ROOTPATH/var/log/kav"
  chmod 1777 "$ROOTPATH/var/log/kav"
  cp -rf /opt/kav "$ROOTPATH/opt/"
  cp -rf /etc/kav "$ROOTPATH/etc"
  cp -rf /var/db/kav "$ROOTPATH/var/db/"
  chmod 666 $ROOTPATH/var/db/kav/5.0/kav4*/keys/application.dat
  samefile "/var/db/kav" "$ROOTPATH/var/db/kav" \
    || echo "Warning: Do not forget to update KAV database in $ROOTPATH !"
fi
if [ "$BOGOFILTER" ]; then
  addbin /usr/bin/bogofilter
  addfile /etc/bogofilter.cf
  echo "bogofilter_dir=/usr/share/bogofilter" >> "$ROOTPATH/etc/bogofilter.cf"
  mkdir -p "$ROOTPATH/usr/share/bogofilter"
  echo "Run: 'chroot $ROOTPATH /usr/bin/bogofilter -s SPAM -n NOSPAM' to learn"
fi
if [ "$QSF" ]; then
  addbin /usr/bin/qsf
  echo "Run: Update for QuickSpamFilter database"
fi

echo "/usr/lib" > "$ROOTPATH/etc/ld.so.conf"
if [ "$BDC" ]; then
  echo "/opt/bdc" >> "$ROOTPATH/etc/ld.so.conf"
fi

# make required devices
rm -f "$ROOTPATH/dev/null" "$ROOTPATH/dev/zero" "$ROOTPATH/dev/urandom"
mknod "$ROOTPATH/dev/null" c 1 3
chmod 0666 "$ROOTPATH/dev/null"
mknod "$ROOTPATH/dev/zero" c 1 5
chmod 0666 "$ROOTPATH/dev/zero"
mknod "$ROOTPATH/dev/urandom" c 1 9
chmod 0444 "$ROOTPATH/dev/urandom"

addbin /lib/libdl.so.2
addbin /lib/libnss_files.so.2

# add archivers
addbin /bin/gzip /usr/bin/gzip
addbin /bin/bzip2 /usr/bin/bzip2
addbin /bin/tar /usr/bin/tar
#addbin /usr/bin/compress /bin/compress
for i in unzip lha arc; do
  [ -x /usr/bin/$i ] && addbin /usr/bin/$i
done
for i in rar arj zoo ace arc; do
  addbin /usr/bin/un$i /usr/bin/$i
done

ldconfig -r "$ROOTPATH"

chmod 1777 "$ROOTPATH/tmp"
chmod 770 "$ROOTPATH/tmp/quarantine"
if [ "`id -u`" = "0" ]; then
  chown -R $USER.$GROUP "$ROOTPATH/tmp" "$ROOTPATH/var/log/sagator"
fi

if [ "$CLAMAV" ]; then
  chgrp clamav "$ROOTPATH/tmp"
fi
